# This file is part of TaskStacker.
#
# TaskStacker is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TaskStacker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TaskStacker.  If not, see <http://www.gnu.org/licenses/>.

import sys

import pygtk
import gtk
import gtk.glade

import resources
import taskfile

class TaskStacker:
    
    def __init__(self):
        self.tasks = []
        self.filename = ""
        self.dirty = False
        self.UI = gtk.Builder()
        self.UI.add_from_file(resources.get_ui_asset("taskstacker.glade"))
        
        maps = {"on_btnPop_clicked"     : self.popTask,
                "on_btnPush_clicked"    : self.pushTask,
                "on_entryTask_changed"  : self.on_entryTask_changed,
                "on_cmdNew_activate"    : self.on_cmdNew_activate,
                "on_cmdOpen_activate"   : self.on_cmdOpen_activate,
                "on_cmdSave_activate"   : self.on_cmdSave_activate,
                "on_cmdSaveAs_activate" : self.on_cmdSaveAs_activate,
                "on_cmdQuit_activate"   : self.quit,
                "on_cmdCut_activate"    : self.on_cmdCut_activate,
                "on_cmdCopy_activate"   : self.on_cmdCopy_activate,
                "on_cmdPaste_activate"  : self.on_cmdPaste_activate,
                "on_cmdDelete_activate" : self.on_cmdDelete_activate,
                "on_mainWindow_destroy" : self.quit,
                "onAbout"               : self.showAbout}
                
        self.UI.connect_signals(maps)
        self.mainwindow = self.UI.get_object("mainWindow")
        self.mainwindow.show_all()
        self.update()
    
    
    def pushTask(self, widget):
        current = self.UI.get_object("entryTask").get_text()
        self.tasks.append(current)
        self.UI.get_object("entryTask").set_text("")
        self.update()
        self.dirty = True
    
    def popTask(self, widget):
        self.tasks.pop()
        self.update()
        self.dirty = True
    
    def on_entryTask_changed(self, widget):
        if len(widget.get_text()):
            self.UI.get_object("btnPush").set_sensitive(True)
        else: # Prevent adding blank lines to the stack
            self.UI.get_object("btnPush").set_sensitive(False)
    
    def on_cmdNew_activate(self, widget):
        if self.dirty == True:
            if not self.confirm():
                return
        self.filename = None
        self.tasks = []
        self.update()
        self.dirty = False
    
    def on_cmdOpen_activate(self, widget):
        if self.dirty == True:
            if not self.confirm():
                return
        path = taskfile.getpath()
        if path:
            self.tasks = taskfile.load_file(path)
            self.filename = path
            self.update()
            self.dirty = False
        else:
            return
    
    def on_cmdSave_activate(self, widget):
        if self.filename:
            taskfile.save_file(self.filename, self.tasks)
            self.dirty = False
        else:
            self.on_cmdSaveAs_activate(widget)
    
    
    def on_cmdSaveAs_activate(self, widget):
        path = taskfile.getpath(True, self.filename)
        if path:
            taskfile.save_file(path, self.tasks)
            self.filename = path
            self.dirty = False

    def on_cmdCut_activate(self, widget):
        entry = self.UI.get_object("entryTask")
        entry.cut_clipboard()
    
    def on_cmdCopy_activate(self, widget):
        entry = self.UI.get_object("entryTask")
        entry.copy_clipboard()
    
    def on_cmdPaste_activate(self, widget):
        entry = self.UI.get_object("entryTask")
        entry.paste_clipboard()
    
    def on_cmdDelete_activate(self, widget):
        entry = self.UI.get_object("entryTask")
        entry.delete_selection()
    
    def update(self):
        """
        Make sure all widgets are in the correct state
        """
        label = self.UI.get_object("labelCurrent")
        popbtn = self.UI.get_object("btnPop")
        if len(self.tasks):
            current = self.tasks[-1]
            label.set_markup("<b>Current Task:</b> " + current)
            popbtn.set_sensitive(True)
        else:
            label.set_markup("<b>Current Task:</b> None")
            popbtn.set_sensitive(False)
        
    
    def showAbout(self, widget):
        aboutdialog = self.UI.get_object("dialogAbout")
        aboutdialog.run()
        aboutdialog.hide()
    
    def quit(self, widget):
        if self.dirty:
            if not self.confirm():
                return
        gtk.main_quit()
    
    def confirm(self):
        msg = gtk.MessageDialog(None,
                                gtk.DIALOG_MODAL,
                                gtk.MESSAGE_WARNING,
                                gtk.BUTTONS_OK_CANCEL,
                                "There are unsaved changes to this task list\n" +
                                "If you continue, you will lose these changes"
                                )
        response = msg.run()
        msg.destroy()
        if response == gtk.RESPONSE_OK:
            return True
        else:
            return False
    