# This file is part of TaskStacker.
#
# TaskStacker is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# TaskStacker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TaskStacker.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import gtk

# We try to use the XDG specification for our data files,
# which should live in ~/.local/share/taskstacker, or whatever
# the specific distro/Locale wants.
# We fallback to hardcoded ~/.local/share/taskstacker if XDG is
# unavailable.
try:
    import xdg.BaseDirectory
    DEFAULT_DIR = os.path.join(xdg.BaseDirectory.xdg_data_home,
                               "taskstacker")
except:
    DEFAULT_DIR = os.path.join(os.path.expanduser("~"), ".local", "share",
                               "taskstacker")

filtertskstk = gtk.FileFilter()
filtertskstk.add_pattern("*.tskstk")
filtertskstk.set_name("TaskStacker Documents")

filterall = gtk.FileFilter()
filterall.add_pattern("*")
filterall.set_name("All Files")


def load_file(path):
    """
    Opens the provided file and returns a list of all lines read
    """
    list = []
    with open(path) as f:
        for line in f:
            list.append(line.rstrip())
    return list

def save_file(path, strlist):
    """
    Writes the provided list as seperate lines in the given file
    """
    with open(path, 'w') as f:
        for line in strlist:
            f.write(line + "\n")

def getpath(action=False, current=None):
    """
    Displays a FileChooserDialog and returns the selected path
    If the first argument is True, we are saving a file.
    If it is False, we are loading a file. Default is False.
    The second argument is the filename to preselect if applicable
    """
    if action:
        if not os.path.exists(DEFAULT_DIR):
            os.mkdir(DEFAULT_DIR)
        dlg = gtk.FileChooserDialog("Save As",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        if current:
            dlg.set_filename(current)
        else:
            dlg.set_current_name("untitled.tskstk")
    else:
        dlg = gtk.FileChooserDialog("Select a file",
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
    
    dlg.set_current_folder(DEFAULT_DIR)
    dlg.add_filter(filterall)
    dlg.add_filter(filtertskstk)
    dlg.set_filter(filtertskstk)
    dlg.set_do_overwrite_confirmation(True)
    response = dlg.run()
    filename = dlg.get_filename()
    dlg.destroy()
    if response == gtk.RESPONSE_OK:
        return(filename)
    return False
        